@@ -484,29 +484,44 @@ void media_object::set_video_frame_template(int index, int width_before_avcodec_
         video_frame_template.chroma_location = video_frame::center;
     }
     // Stereo layout
-    video_frame_template.stereo_layout = video_frame::mono;
+    video_frame_template.stereo_layout = parameters::layout_mono;
     video_frame_template.stereo_layout_swap = false;
     std::string val;
     /* Determine the stereo layout from the resolution.*/
     if (video_frame_template.raw_width / 2 > video_frame_template.raw_height)
     {
-        video_frame_template.stereo_layout = video_frame::left_right;
+        video_frame_template.stereo_layout = parameters::layout_left_right;
     }
     else if (video_frame_template.raw_height > video_frame_template.raw_width)
     {
-        video_frame_template.stereo_layout = video_frame::top_bottom;
+        video_frame_template.stereo_layout = parameters::layout_top_bottom;
     }
-    /* MPO files are alternating-left-right. */
-    if (_url.length() > 4
-            && (_url.substr(_url.length() - 4) == ".mpo"
-                || _url.substr(_url.length() - 4) == ".MPO"))
+    /* Gather hints from the filename extension */
+    std::string extension;
+    size_t extension_dot_pos = _url.find_last_of('.');
+    if (extension_dot_pos != std::string::npos)
     {
-        video_frame_template.stereo_layout = video_frame::alternating;
+        extension = _url.substr(extension_dot_pos + 1);
+        for (size_t i = 0; i < extension.length(); i++)
+        {
+            extension[i] =  std::tolower(extension[i]);
+        }
+    }
+    if (extension == "mpo")
+    {
+        /* MPO files are alternating-left-right. */
+        video_frame_template.stereo_layout = parameters::layout_alternating;
+    }
+    else if (extension == "jps" || extension == "pns")
+    {
+        /* JPS and PNS are side-by-side in right-left mode */
+        video_frame_template.stereo_layout = parameters::layout_left_right;
+        video_frame_template.stereo_layout_swap = true;
     }
     /* Determine the input mode by looking at the file name.
      * This should be compatible to these conventions:
      * http://www.tru3d.com/technology/3D_Media_Formats_Software.php?file=TriDef%20Supported%203D%20Formats */
-    std::string marker = _url.substr(0, _url.find_last_of('.'));
+    std::string marker = _url.substr(0, extension_dot_pos);
     size_t last_dash = marker.find_last_of('-');
     if (last_dash != std::string::npos)
     {
@@ -522,69 +537,69 @@ void media_object::set_video_frame_template(int index, int width_before_avcodec_
     }
     if (marker == "lr")
     {
-        video_frame_template.stereo_layout = video_frame::left_right;
+        video_frame_template.stereo_layout = parameters::layout_left_right;
         video_frame_template.stereo_layout_swap = false;
     }
     else if (marker == "rl")
     {
-        video_frame_template.stereo_layout = video_frame::left_right;
+        video_frame_template.stereo_layout = parameters::layout_left_right;
         video_frame_template.stereo_layout_swap = true;
     }
     else if (marker == "lrh" || marker == "lrq")
     {
-        video_frame_template.stereo_layout = video_frame::left_right_half;
+        video_frame_template.stereo_layout = parameters::layout_left_right_half;
         video_frame_template.stereo_layout_swap = false;
     }
     else if (marker == "rlh" || marker == "rlq")
     {
-        video_frame_template.stereo_layout = video_frame::left_right_half;
+        video_frame_template.stereo_layout = parameters::layout_left_right_half;
         video_frame_template.stereo_layout_swap = true;
     }
     else if (marker == "tb" || marker == "ab")
     {
-        video_frame_template.stereo_layout = video_frame::top_bottom;
+        video_frame_template.stereo_layout = parameters::layout_top_bottom;
         video_frame_template.stereo_layout_swap = false;
     }
     else if (marker == "bt" || marker == "ba")
     {
-        video_frame_template.stereo_layout = video_frame::top_bottom;
+        video_frame_template.stereo_layout = parameters::layout_top_bottom;
         video_frame_template.stereo_layout_swap = true;
     }
     else if (marker == "tbh" || marker == "abq")
     {
-        video_frame_template.stereo_layout = video_frame::top_bottom_half;
+        video_frame_template.stereo_layout = parameters::layout_top_bottom_half;
         video_frame_template.stereo_layout_swap = false;
     }
     else if (marker == "bth" || marker == "baq")
     {
-        video_frame_template.stereo_layout = video_frame::top_bottom_half;
+        video_frame_template.stereo_layout = parameters::layout_top_bottom_half;
         video_frame_template.stereo_layout_swap = true;
     }
     else if (marker == "eo")
     {
-        video_frame_template.stereo_layout = video_frame::even_odd_rows;
+        video_frame_template.stereo_layout = parameters::layout_even_odd_rows;
         video_frame_template.stereo_layout_swap = false;
         // all image lines are given in this case, and there should be no interpolation [TODO]
     }
     else if (marker == "oe")
     {
-        video_frame_template.stereo_layout = video_frame::even_odd_rows;
+        video_frame_template.stereo_layout = parameters::layout_even_odd_rows;
         video_frame_template.stereo_layout_swap = true;
         // all image lines are given in this case, and there should be no interpolation [TODO]
     }
     else if (marker == "eoq" || marker == "3dir")
     {
-        video_frame_template.stereo_layout = video_frame::even_odd_rows;
+        video_frame_template.stereo_layout = parameters::layout_even_odd_rows;
         video_frame_template.stereo_layout_swap = false;
     }
     else if (marker == "oeq" || marker == "3di")
     {
-        video_frame_template.stereo_layout = video_frame::even_odd_rows;
+        video_frame_template.stereo_layout = parameters::layout_even_odd_rows;
         video_frame_template.stereo_layout_swap = true;
     }
     else if (marker == "2d")
     {
-        video_frame_template.stereo_layout = video_frame::mono;
+        video_frame_template.stereo_layout = parameters::layout_mono;
         video_frame_template.stereo_layout_swap = false;
     }
     /* Check some tags defined at this link: http://www.3dtv.at/Knowhow/StereoWmvSpec_en.aspx
@@ -594,14 +609,14 @@ void media_object::set_video_frame_template(int index, int width_before_avcodec_
     {
         video_frame_template.stereo_layout_swap = (val == "SideBySideRF");
         val = tag_value("StereoscopicHalfWidth");
-        video_frame_template.stereo_layout = (val == "1" ? video_frame::left_right_half : video_frame::left_right);
+        video_frame_template.stereo_layout = (val == "1" ? parameters::layout_left_right_half : parameters::layout_left_right);
 
     }
     else if (val == "OverUnderRT" || val == "OverUnderLT")
     {
         video_frame_template.stereo_layout_swap = (val == "OverUnderRT");
         val = tag_value("StereoscopicHalfHeight");
-        video_frame_template.stereo_layout = (val == "1" ? video_frame::top_bottom_half : video_frame::top_bottom);
+        video_frame_template.stereo_layout = (val == "1" ? parameters::layout_top_bottom_half : parameters::layout_top_bottom);
     }
     /* Check the Matroska StereoMode metadata, which is translated by FFmpeg to a "stereo_mode" tag.
      * This tag is per-track, not per-file!
@@ -620,18 +635,18 @@ void media_object::set_video_frame_template(int index, int width_before_avcodec_
     }
     if (val == "mono")
     {
-        video_frame_template.stereo_layout = video_frame::mono;
+        video_frame_template.stereo_layout = parameters::layout_mono;
         video_frame_template.stereo_layout_swap = false;
     }
     else if (val == "left_right" || val == "right_left")
     {
         if (video_frame_template.raw_width / 2 > video_frame_template.raw_height)
         {
-            video_frame_template.stereo_layout = video_frame::left_right;
+            video_frame_template.stereo_layout = parameters::layout_left_right;
         }
         else
         {
-            video_frame_template.stereo_layout = video_frame::left_right_half;
+            video_frame_template.stereo_layout = parameters::layout_left_right_half;
         }
         video_frame_template.stereo_layout_swap = (val == "right_left");
     }
@@ -639,42 +654,42 @@ void media_object::set_video_frame_template(int index, int width_before_avcodec_
     {
         if (video_frame_template.raw_height > video_frame_template.raw_width)
         {
-            video_frame_template.stereo_layout = video_frame::top_bottom;
+            video_frame_template.stereo_layout = parameters::layout_top_bottom;
         }
         else
         {
-            video_frame_template.stereo_layout = video_frame::top_bottom_half;
+            video_frame_template.stereo_layout = parameters::layout_top_bottom_half;
         }
         video_frame_template.stereo_layout_swap = (val == "bottom_top");
     }
     else if (val == "row_interleaved_lr" || val == "row_interleaved_rl")
     {
-        video_frame_template.stereo_layout = video_frame::even_odd_rows;
+        video_frame_template.stereo_layout = parameters::layout_even_odd_rows;
         video_frame_template.stereo_layout_swap = (val == "row_interleaved_rl");
     }
     else if (val == "block_lr" || val == "block_rl")
     {
-        video_frame_template.stereo_layout = video_frame::alternating;
+        video_frame_template.stereo_layout = parameters::layout_alternating;
         video_frame_template.stereo_layout_swap = (val == "block_rl");
     }
     else if (!val.empty())
     {
         msg::wrn(_("%s video stream %d: Unsupported stereo layout %s."),
                 _url.c_str(), index + 1, str::sanitize(val).c_str());
-        video_frame_template.stereo_layout = video_frame::mono;
+        video_frame_template.stereo_layout = parameters::layout_mono;
         video_frame_template.stereo_layout_swap = false;
     }
     /* Sanity checks. If these fail, use safe fallback */
-    if (((video_frame_template.stereo_layout == video_frame::left_right
-                    || video_frame_template.stereo_layout == video_frame::left_right_half)
+    if (((video_frame_template.stereo_layout == parameters::layout_left_right
+                    || video_frame_template.stereo_layout == parameters::layout_left_right_half)
                 && video_frame_template.raw_width % 2 != 0)
-            || ((video_frame_template.stereo_layout == video_frame::top_bottom
-                    || video_frame_template.stereo_layout == video_frame::top_bottom_half)
+            || ((video_frame_template.stereo_layout == parameters::layout_top_bottom
+                    || video_frame_template.stereo_layout == parameters::layout_top_bottom_half)
                 && video_frame_template.raw_height % 2 != 0)
-            || (video_frame_template.stereo_layout == video_frame::even_odd_rows
+            || (video_frame_template.stereo_layout == parameters::layout_even_odd_rows
                 && video_frame_template.raw_height % 2 != 0))
     {
-        video_frame_template.stereo_layout = video_frame::mono;
+        video_frame_template.stereo_layout = parameters::layout_mono;
         video_frame_template.stereo_layout_swap = false;
     }
     /* Set width and height of a single view */
@@ -832,7 +847,8 @@ void media_object::open(const std::string &url, const device_request &dev_reques
     {
         _ffmpeg->format_ctx->streams[i]->discard = AVDISCARD_ALL;        // ignore by default; user must activate streams
         AVCodecContext *codec_ctx = _ffmpeg->format_ctx->streams[i]->codec;
-        AVCodec *codec = NULL;
+        AVCodec *codec = (codec_ctx->codec_id == CODEC_ID_TEXT
+                ? NULL : avcodec_find_decoder(codec_ctx->codec_id));
         // XXX: Sometimes the reported width and height for a video stream change after avcodec_open(),
         // but the original values seem to be correct. This seems to happen mostly with 1920x1080 video
         // that later is reported as 1920x1088, which results in a gray bar displayed at the bottom of
@@ -840,25 +856,28 @@ void media_object::open(const std::string &url, const device_request &dev_reques
         // them later in set_video_frame_template().
         int width_before_avcodec_open = codec_ctx->width;
         int height_before_avcodec_open = codec_ctx->height;
-        if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
+        if (codec_ctx->codec_type == AVMEDIA_TYPE_VIDEO)
         {
             // Activate multithreaded decoding. This must be done before opening the codec; see
             // http://lists.gnu.org/archive/html/bino-list/2011-08/msg00019.html
             codec_ctx->thread_count = video_decoding_threads();
+            // Set CODEC_FLAG_EMU_EDGE in the same situations in which ffplay sets it.
+            // I don't know what exactly this does, but it is necessary to fix the problem
+            // described in this thread: http://lists.nongnu.org/archive/html/bino-list/2012-02/msg00039.html
+            if (codec_ctx->lowres || (codec && (codec->capabilities & CODEC_CAP_DR1)))
+                codec_ctx->flags |= CODEC_FLAG_EMU_EDGE;
         }
         // Find and open the codec. CODEC_ID_TEXT is a special case: it has no decoder since it is unencoded raw data.
-        if (_ffmpeg->format_ctx->streams[i]->codec->codec_id != CODEC_ID_TEXT
-                && (!(codec = avcodec_find_decoder(_ffmpeg->format_ctx->streams[i]->codec->codec_id))
-                    || (e = avcodec_open(codec_ctx, codec)) < 0))
+        if (codec_ctx->codec_id != CODEC_ID_TEXT && (!codec || (e = avcodec_open(codec_ctx, codec)) < 0))
         {
             msg::wrn(_("%s stream %d: Cannot open %s: %s"), _url.c_str(), i,
-                    _ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO ? _("video codec")
-                    : _ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO ? _("audio codec")
-                    : _ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_SUBTITLE ? _("subtitle codec")
+                    codec_ctx->codec_type == AVMEDIA_TYPE_VIDEO ? _("video codec")
+                    : codec_ctx->codec_type == AVMEDIA_TYPE_AUDIO ? _("audio codec")
+                    : codec_ctx->codec_type == AVMEDIA_TYPE_SUBTITLE ? _("subtitle codec")
                     : _("data"),
                     codec ? my_av_strerror(e).c_str() : _("codec not supported"));
         }
-        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
+        else if (codec_ctx->codec_type == AVMEDIA_TYPE_VIDEO)
         {
             _ffmpeg->video_streams.push_back(i);
             int j = _ffmpeg->video_streams.size() - 1;
@@ -922,7 +941,7 @@ void media_object::open(const std::string &url, const device_request &dev_reques
             }
             _ffmpeg->video_last_timestamps.push_back(std::numeric_limits<int64_t>::min());
         }
-        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
+        else if (codec_ctx->codec_type == AVMEDIA_TYPE_AUDIO)
         {
             _ffmpeg->audio_streams.push_back(i);
             int j = _ffmpeg->audio_streams.size() - 1;
@@ -943,7 +962,7 @@ void media_object::open(const std::string &url, const device_request &dev_reques
             _ffmpeg->audio_buffers.push_back(std::vector<unsigned char>());
             _ffmpeg->audio_last_timestamps.push_back(std::numeric_limits<int64_t>::min());
         }
-        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_SUBTITLE)
+        else if (codec_ctx->codec_type == AVMEDIA_TYPE_SUBTITLE)
         {
             _ffmpeg->subtitle_streams.push_back(i);
             int j = _ffmpeg->subtitle_streams.size() - 1;
@@ -1382,6 +1401,7 @@ void video_decode_thread::run()
     for (int raw_frame = 0; raw_frame < _raw_frames; raw_frame++)
     {
         int frame_finished = 0;
+read_frame:
         do
         {
             bool empty;
@@ -1426,7 +1446,15 @@ void video_decode_thread::run()
                     &(_ffmpeg->video_packets[_video_stream]));
         }
         while (!frame_finished);
-
+#if LIBAVCODEC_VERSION_MAJOR >= 53 && LIBAVCODEC_VERSION_MINOR >= 8
+        if (_ffmpeg->video_frames[_video_stream]->width != _ffmpeg->video_frame_templates[_video_stream].raw_width
+                || _ffmpeg->video_frames[_video_stream]->height != _ffmpeg->video_frame_templates[_video_stream].raw_height)
+        {
+            msg::wrn(_("%s video stream %d: Dropping %dx%d frame"), _url.c_str(), _video_stream + 1, 
+                    _ffmpeg->video_frames[_video_stream]->width, _ffmpeg->video_frames[_video_stream]->height);
+            goto read_frame;
+        }
+#endif
         if (_frame.layout == video_frame::bgra32)
         {
             sws_scale(_ffmpeg->video_sws_ctxs[_video_stream],
