@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Stefan Eilemann <eile@eyescale.ch>
  *
@@ -34,6 +34,8 @@
 #include "msg.h"
 #include "s11n.h"
 
+#include "dispatch.h"
+#include "media_input.h"
 #include "video_output.h"
 #include "player_equalizer.h"
 
@@ -55,6 +57,7 @@
  * the video.
  */
 
+extern dispatch* global_dispatch;
 
 /*
  * player_eq_node
@@ -66,50 +69,25 @@
 class player_eq_node : public player
 {
 private:
-    bool _is_master;
-
-protected:
-    video_output *create_video_output()
-    {
-        // A node does not have its own video output; this is handled via eq_window.
-        return NULL;
-    }
-
-    void destory_video_output(video_output *)
-    {
-    }
-
-    audio_output *create_audio_output()
-    {
-        // Only the master player may have an audio output.
-        return _is_master ? new audio_output() : NULL;
-    }
-
-    void destroy_audio_output(audio_output *ao)
-    {
-        delete ao;
-    }
+    class media_input _media_input;
 
 public:
-    player_eq_node() : player(player::slave), _is_master(false)
+    player_eq_node() : player()
     {
     }
 
-    void make_master()
+    bool init(const open_input_data& input)
     {
-        player::make_master();
-        _is_master = true;
-    }
-
-    bool init(const player_init_data &init_data, video_frame &frame_template)
-    {
-        try
-        {
-            player::open(init_data);
-            frame_template = get_media_input().video_frame_template();
+        try {
+            _media_input.open(input.urls, input.dev_request);
+            if (_media_input.video_streams() == 0)
+                throw exc(_("No video streams found."));
+            if (input.params.stereo_layout_is_set() || input.params.stereo_layout_swap_is_set())
+                _media_input.set_stereo_layout(input.params.stereo_layout(), input.params.stereo_layout_swap());
+            _media_input.select_video_stream(input.params.video_stream());
+            player::open();
         }
-        catch (std::exception &e)
-        {
+        catch (std::exception& e) {
             msg::err("%s", e.what());
             return false;
         }
@@ -118,7 +96,7 @@ public:
 
     void seek(int64_t pos)
     {
-        get_media_input_nonconst().seek(pos);
+        _media_input.seek(pos);
         // The master player read a video frame; do the same to keep sync
         start_frame_read();
     }
@@ -126,35 +104,19 @@ public:
     void start_frame_read()
     {
         // Only called on slave nodes
-        get_media_input_nonconst().start_video_frame_read();
+        _media_input.start_video_frame_read();
     }
 
     void finish_frame_read()
     {
         // Only called on slave nodes
-        _video_frame = get_media_input_nonconst().finish_video_frame_read();
+        _video_frame = _media_input.finish_video_frame_read();
         if (!_video_frame.is_valid())
         {
             msg::err(_("Reading input frame failed."));
             std::exit(1);
         }
     }
-
-    void step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool *drop_frame, bool *display_frame)
-    {
-        // Only called on the master node
-        player::step(more_steps, seek_to, prep_frame, drop_frame, display_frame);
-    }
-
-    const video_frame &get_video_frame()
-    {
-        return _video_frame;
-    }
-
-    const subtitle_box &get_subtitle()
-    {
-        return _current_subtitle_box;
-    }
 };
 
 /*
@@ -176,12 +138,24 @@ private:
     float _canvas_video_area_h;
 
 protected:
-    int video_display_width() { return screen_width() * _canvas_video_area_w; }
-    int video_display_height() { return screen_height() * _canvas_video_area_h; }
+    int video_display_width() const { return screen_width() * _canvas_video_area_w; }
+    int video_display_height() const { return screen_height() * _canvas_video_area_h; }
+    int screen_width() const { return _channel->getPixelViewport().w / _channel->getViewport().w; }
+    int screen_height() const { return _channel->getPixelViewport().h / _channel->getViewport().h; }
+    float screen_pixel_aspect_ratio() const
+    {
+        float pixels_per_unit_x = _channel->getPixelViewport().w / (_canvas_width * _channel->getViewport().w);
+        float pixels_per_unit_y = _channel->getPixelViewport().h / (_canvas_height * _channel->getViewport().h);
+        return pixels_per_unit_y / pixels_per_unit_x;
+    }
+    int width() const { return _channel->getPixelViewport().w; }
+    int height() const { return _channel->getPixelViewport().h; }
+    int pos_x() const { return 0; }
+    int pos_y() const { return 0; }
+    GLEWContext* glewGetContext() const { return const_cast<GLEWContext*>(_channel->getWindow()->glewGetContext()); }
     void make_context_current() { _channel->getWindow()->makeCurrent(); }
-    bool context_is_stereo() { return false; }
+    bool context_is_stereo() const { return false; }
     void recreate_context(bool) { }
-    void trigger_update() { }
     void trigger_resize(int, int) { }
 
 public:
@@ -206,26 +180,10 @@ public:
         return 0;
     }
     bool supports_stereo() const { return false; }
-    int screen_width() { return _channel->getPixelViewport().w / _channel->getViewport().w; }
-    int screen_height() { return _channel->getPixelViewport().h / _channel->getViewport().h; }
-    float screen_pixel_aspect_ratio()
-    {
-        float pixels_per_unit_x = _channel->getPixelViewport().w / (_canvas_width * _channel->getViewport().w);
-        float pixels_per_unit_y = _channel->getPixelViewport().h / (_canvas_height * _channel->getViewport().h);
-        return pixels_per_unit_y / pixels_per_unit_x;
-    }
-    int width() { return _channel->getPixelViewport().w; }
-    int height() { return _channel->getPixelViewport().h; }
-    int pos_x() { return 0; }
-    int pos_y() { return 0; }
-    bool fullscreen() { return false; }
     void center() { }
-    void enter_fullscreen(int) { }
+    void enter_fullscreen() { }
     void exit_fullscreen() { }
-    bool toggle_fullscreen(int) { return false; }
-    bool has_events() { return false; }
     void process_events() { }
-    void receive_notification(const notification &) { }
 
 public:
     video_output_eq_channel(eq::Channel *channel,
@@ -265,7 +223,7 @@ public:
         GLint vp[2][4];
         std::memcpy(vp[0], viewport, 4 * sizeof(int));
         std::memcpy(vp[1], viewport, 4 * sizeof(int));
-        video_output::display_current_frame(true, mono_right_instead_of_left,
+        video_output::display_current_frame(0, true, mono_right_instead_of_left,
                 x, y, w, h, vp, my_tex_coords);
     }
 };
@@ -323,12 +281,13 @@ class eq_init_data : public co::Object
 {
 public:
     eq::uint128_t frame_data_id;
-    player_init_data init_data;
+    open_input_data input;
+    parameters params;
     bool flat_screen;
     float canvas_width;
     float canvas_height;
 
-    eq_init_data() : init_data()
+    eq_init_data()
     {
         flat_screen = true;
     }
@@ -348,7 +307,8 @@ protected:
         std::ostringstream oss;
         s11n::save(oss, frame_data_id.high());
         s11n::save(oss, frame_data_id.low());
-        s11n::save(oss, init_data);
+        s11n::save(oss, input);
+        s11n::save(oss, params);
         s11n::save(oss, flat_screen);
         s11n::save(oss, canvas_width);
         s11n::save(oss, canvas_height);
@@ -362,7 +322,8 @@ protected:
         std::istringstream iss(s);
         s11n::load(iss, frame_data_id.high());
         s11n::load(iss, frame_data_id.low());
-        s11n::load(iss, init_data);
+        s11n::load(iss, input);
+        s11n::load(iss, params);
         s11n::load(iss, flat_screen);
         s11n::load(iss, canvas_width);
         s11n::load(iss, canvas_height);
@@ -376,23 +337,23 @@ protected:
 class eq_frame_data : public co::Object
 {
 public:
+    std::string dispatch_state;
     subtitle_box subtitle;
-    parameters params;
     int64_t seek_to;
     bool prep_frame;
     bool drop_frame;
     bool display_frame;
+    bool display_statistics;
     struct { float x, y, w, h, d; } canvas_video_area;
     float tex_coords[4][2];
 
 public:
     eq_frame_data() :
-        subtitle(),
-        params(),
         seek_to(0),
         prep_frame(false),
         drop_frame(false),
-        display_frame(false)
+        display_frame(false),
+        display_statistics(false)
     {
     }
 
@@ -405,12 +366,13 @@ protected:
     virtual void getInstanceData(co::DataOStream &os)
     {
         std::ostringstream oss;
+        s11n::save(oss, dispatch_state);
         s11n::save(oss, subtitle);
-        s11n::save(oss, params);
         s11n::save(oss, seek_to);
         s11n::save(oss, prep_frame);
         s11n::save(oss, drop_frame);
         s11n::save(oss, display_frame);
+        s11n::save(oss, display_statistics);
         s11n::save(oss, &canvas_video_area, sizeof(canvas_video_area));
         s11n::save(oss, tex_coords, sizeof(tex_coords));
         os << oss.str();
@@ -421,12 +383,13 @@ protected:
         std::string s;
         is >> s;
         std::istringstream iss(s);
+        s11n::load(iss, dispatch_state);
         s11n::load(iss, subtitle);
-        s11n::load(iss, params);
         s11n::load(iss, seek_to);
         s11n::load(iss, prep_frame);
         s11n::load(iss, drop_frame);
         s11n::load(iss, display_frame);
+        s11n::load(iss, display_statistics);
         s11n::load(iss, &canvas_video_area, sizeof(canvas_video_area));
         s11n::load(iss, tex_coords, sizeof(tex_coords));
     }
@@ -442,21 +405,9 @@ private:
     bool _is_master_config;
     eq_init_data _eq_init_data;         // Master eq_init_data instance
     eq_frame_data _eq_frame_data;       // Master eq_frame_data instance
-    player_eq_node _player;             // Master player
-    controller _controller;             // Sends commands to the player
 
 public:
-    video_frame frame_template;         // Video frame properties
-
-public:
-    eq_config(eq::ServerPtr parent) :
-        eq::Config(parent),
-        _is_master_config(false),
-        _eq_init_data(),
-        _eq_frame_data(),
-        _player(),
-        _controller(),
-        frame_template()
+    eq_config(eq::ServerPtr parent) : eq::Config(parent), _is_master_config(false)
     {
     }
 
@@ -465,22 +416,15 @@ public:
         return _is_master_config;
     }
 
-    bool init(const player_init_data &init_data, bool flat_screen)
+    bool init(const open_input_data& input, bool flat_screen)
     {
-        msg::set_level(init_data.log_level);
         msg::dbg(HERE);
         // If this function is called, then this is the master config
         _is_master_config = true;
         // Initialize master init/frame data instances
-        _eq_init_data.init_data = init_data;
+        _eq_init_data.input = input;
+        _eq_init_data.params = dispatch::parameters();
         _eq_init_data.flat_screen = flat_screen;
-        _eq_frame_data.params = _eq_init_data.init_data.params;
-        // Initialize master player
-        _player.make_master();
-        if (!_player.init(init_data, frame_template))
-        {
-            return false;
-        }
         // Find canvas
         if (getCanvases().size() < 1)
         {
@@ -507,70 +451,86 @@ public:
         // Deregister master instances
         deregisterObject(&_eq_init_data);
         deregisterObject(&_eq_frame_data);
-        // Cleanup
-        _player.close();
         msg::dbg(HERE);
         return ret;
     }
 
     virtual uint32_t startFrame()
     {
-        // Run one player step to find out what to do
+        // Run player steps until we are told to do something
         bool more_steps;
-        _player.step(&more_steps, &_eq_frame_data.seek_to,
-                &_eq_frame_data.prep_frame, &_eq_frame_data.drop_frame, &_eq_frame_data.display_frame);
+        do {
+            global_dispatch->get_player()->step(&more_steps, &_eq_frame_data.seek_to,
+                    &_eq_frame_data.prep_frame, &_eq_frame_data.drop_frame, &_eq_frame_data.display_frame);
+            dispatch::process_all_events();
+        }
+        while (more_steps
+                && _eq_frame_data.seek_to == -1
+                && !_eq_frame_data.prep_frame
+                && !_eq_frame_data.drop_frame
+                && !_eq_frame_data.display_frame);
         if (!more_steps)
         {
             this->exit();
         }
         // Update the video state for all (it might have changed via handleEvent())
-        _eq_frame_data.subtitle = _player.get_subtitle();
-        _eq_frame_data.params = _player.get_parameters();
+        _eq_frame_data.subtitle = global_dispatch->get_player()->get_subtitle_box();
+        _eq_frame_data.dispatch_state = global_dispatch->save_state();
         // Find region of canvas to use, depending on the video aspect ratio and zoom level
+        float aspect_ratio = dispatch::media_input()->video_frame_template().aspect_ratio;
+        float crop_aspect_ratio = dispatch::parameters().crop_aspect_ratio();
         float canvas_aspect_ratio = _eq_init_data.canvas_width / _eq_init_data.canvas_height;
-        if (_eq_init_data.flat_screen)
-        {
-            if (frame_template.aspect_ratio >= canvas_aspect_ratio)
-            {
+        float zoom = dispatch::parameters().zoom();
+        if (_eq_init_data.flat_screen) {
+            _eq_frame_data.tex_coords[0][0] = 0.0f;
+            _eq_frame_data.tex_coords[0][1] = 0.0f;
+            _eq_frame_data.tex_coords[1][0] = 1.0f;
+            _eq_frame_data.tex_coords[1][1] = 0.0f;
+            _eq_frame_data.tex_coords[2][0] = 1.0f;
+            _eq_frame_data.tex_coords[2][1] = 1.0f;
+            _eq_frame_data.tex_coords[3][0] = 0.0f;
+            _eq_frame_data.tex_coords[3][1] = 1.0f;
+            if (crop_aspect_ratio > 0.0f) {
+                if (aspect_ratio >= crop_aspect_ratio) {
+                    float cutoff = (1.0f - crop_aspect_ratio / aspect_ratio) / 2.0f;
+                    _eq_frame_data.tex_coords[0][0] += cutoff;
+                    _eq_frame_data.tex_coords[1][0] -= cutoff;
+                    _eq_frame_data.tex_coords[2][0] -= cutoff;
+                    _eq_frame_data.tex_coords[3][0] += cutoff;
+                } else {
+                    float cutoff = (1.0f - aspect_ratio / crop_aspect_ratio) / 2.0f;
+                    _eq_frame_data.tex_coords[0][1] += cutoff;
+                    _eq_frame_data.tex_coords[1][1] += cutoff;
+                    _eq_frame_data.tex_coords[2][1] -= cutoff;
+                    _eq_frame_data.tex_coords[3][1] -= cutoff;
+                }
+                aspect_ratio = crop_aspect_ratio;
+            }
+            if (aspect_ratio >= canvas_aspect_ratio) {
                 // need black borders top and bottom
-                float zoom_src_ar = _eq_frame_data.params.zoom * canvas_aspect_ratio
-                    + (1.0f - _eq_frame_data.params.zoom) * frame_template.aspect_ratio;
+                float zoom_aspect_ratio = zoom * canvas_aspect_ratio + (1.0f - zoom) * aspect_ratio;
                 _eq_frame_data.canvas_video_area.w = 1.0f;
-                _eq_frame_data.canvas_video_area.h = canvas_aspect_ratio / zoom_src_ar;
+                _eq_frame_data.canvas_video_area.h = canvas_aspect_ratio / zoom_aspect_ratio;
                 _eq_frame_data.canvas_video_area.x = (1.0f - _eq_frame_data.canvas_video_area.w) / 2.0f;
                 _eq_frame_data.canvas_video_area.y = (1.0f - _eq_frame_data.canvas_video_area.h) / 2.0f;
-                float cutoff = (1.0f - zoom_src_ar / frame_template.aspect_ratio) / 2.0f;
-                _eq_frame_data.tex_coords[0][0] = cutoff;
-                _eq_frame_data.tex_coords[0][1] = 0.0f;
-                _eq_frame_data.tex_coords[1][0] = 1.0f - cutoff;
-                _eq_frame_data.tex_coords[1][1] = 0.0f;
-                _eq_frame_data.tex_coords[2][0] = 1.0f - cutoff;
-                _eq_frame_data.tex_coords[2][1] = 1.0f;
-                _eq_frame_data.tex_coords[3][0] = cutoff;
-                _eq_frame_data.tex_coords[3][1] = 1.0f;
-            }
-            else
-            {
+                float cutoff = (1.0f - zoom_aspect_ratio / aspect_ratio) / 2.0f;
+                _eq_frame_data.tex_coords[0][0] += cutoff;
+                _eq_frame_data.tex_coords[1][0] -= cutoff;
+                _eq_frame_data.tex_coords[2][0] -= cutoff;
+                _eq_frame_data.tex_coords[3][0] += cutoff;
+            } else {
                 // need black borders left and right
-                _eq_frame_data.canvas_video_area.w = frame_template.aspect_ratio / canvas_aspect_ratio;
+                _eq_frame_data.canvas_video_area.w = aspect_ratio / canvas_aspect_ratio;
                 _eq_frame_data.canvas_video_area.h = 1.0f;
                 _eq_frame_data.canvas_video_area.x = (1.0f - _eq_frame_data.canvas_video_area.w) / 2.0f;
                 _eq_frame_data.canvas_video_area.y = (1.0f - _eq_frame_data.canvas_video_area.h) / 2.0f;
-                _eq_frame_data.tex_coords[0][0] = 0.0f;
-                _eq_frame_data.tex_coords[0][1] = 0.0f;
-                _eq_frame_data.tex_coords[1][0] = 1.0f;
-                _eq_frame_data.tex_coords[1][1] = 0.0f;
-                _eq_frame_data.tex_coords[2][0] = 1.0f;
-                _eq_frame_data.tex_coords[2][1] = 1.0f;
-                _eq_frame_data.tex_coords[3][0] = 0.0f;
-                _eq_frame_data.tex_coords[3][1] = 1.0f;
             }
         }
         else
         {
             compute_3d_canvas(&_eq_frame_data.canvas_video_area.h, &_eq_frame_data.canvas_video_area.d);
             // compute width and offset for 1m high 'screen' quad in 3D space
-            _eq_frame_data.canvas_video_area.w = _eq_frame_data.canvas_video_area.h * frame_template.aspect_ratio;
+            _eq_frame_data.canvas_video_area.w = _eq_frame_data.canvas_video_area.h * aspect_ratio;
             _eq_frame_data.canvas_video_area.x = -0.5f * _eq_frame_data.canvas_video_area.w;
             _eq_frame_data.canvas_video_area.y = -0.5f * _eq_frame_data.canvas_video_area.h;
             _eq_frame_data.tex_coords[0][0] = 0.0f;
@@ -598,11 +558,14 @@ public:
         {
             switch (event->data.keyPress.key)
             {
+            case 'S':
+                _eq_frame_data.display_statistics = !_eq_frame_data.display_statistics;
+                break;
             case 'q':
-                _controller.send_cmd(command::toggle_play);
+                controller::send_cmd(command::toggle_play);
                 break;
             case 'e':
-                _controller.send_cmd(command::toggle_stereo_mode_swap);
+                controller::send_cmd(command::toggle_stereo_mode_swap);
                 break;
             case 'f':
                 /* fullscreen toggling not supported with Equalizer */
@@ -612,85 +575,85 @@ public:
                 break;
             case ' ':
             case 'p':
-                _controller.send_cmd(command::toggle_pause);
+                controller::send_cmd(command::toggle_pause);
                 break;
             case 'v':
                 /* TODO: cycling video streams is currently not supported with Equalizer.
                  * We would have to cycle the streams in all node players, and thus communicate
                  * the change via frame data. */
-                //_controller.send_cmd(command::cycle_video_stream);
+                //controller::send_cmd(command::cycle_video_stream);
                 break;
             case 'a':
                 /* TODO: cycling audio streams is currently not supported with Equalizer.
                  * We would have to cycle the streams in all node players, and thus communicate
                  * the change via frame data. */
-                //_controller.send_cmd(command::cycle_audio_stream);
+                //controller::send_cmd(command::cycle_audio_stream);
                 break;
             case 's':
                 /* TODO: cycling subtitle streams is currently not supported with Equalizer.
                  * We would have to cycle the streams in all node players, and thus communicate
                  * the change via frame data. */
-                //_controller.send_cmd(command::cycle_subtitle_stream);
+                //controller::send_cmd(command::cycle_subtitle_stream);
                 break;
             case '1':
-                _controller.send_cmd(command::adjust_contrast, -0.05f);
+                controller::send_cmd(command::adjust_contrast, -0.05f);
                 break;
             case '2':
-                _controller.send_cmd(command::adjust_contrast, +0.05f);
+                controller::send_cmd(command::adjust_contrast, +0.05f);
                 break;
             case '3':
-                _controller.send_cmd(command::adjust_brightness, -0.05f);
+                controller::send_cmd(command::adjust_brightness, -0.05f);
                 break;
             case '4':
-                _controller.send_cmd(command::adjust_brightness, +0.05f);
+                controller::send_cmd(command::adjust_brightness, +0.05f);
                 break;
             case '5':
-                _controller.send_cmd(command::adjust_hue, -0.05f);
+                controller::send_cmd(command::adjust_hue, -0.05f);
                 break;
             case '6':
-                _controller.send_cmd(command::adjust_hue, +0.05f);
+                controller::send_cmd(command::adjust_hue, +0.05f);
                 break;
             case '7':
-                _controller.send_cmd(command::adjust_saturation, -0.05f);
+                controller::send_cmd(command::adjust_saturation, -0.05f);
                 break;
             case '8':
-                _controller.send_cmd(command::adjust_saturation, +0.05f);
+                controller::send_cmd(command::adjust_saturation, +0.05f);
                 break;
             case '[':
-                _controller.send_cmd(command::adjust_parallax, -0.01f);
+                controller::send_cmd(command::adjust_parallax, -0.01f);
                 break;
             case ']':
-                _controller.send_cmd(command::adjust_parallax, +0.01f);
+                controller::send_cmd(command::adjust_parallax, +0.01f);
                 break;
             case '(':
-                _controller.send_cmd(command::adjust_ghostbust, -0.01f);
+                controller::send_cmd(command::adjust_ghostbust, -0.01f);
                 break;
             case ')':
-                _controller.send_cmd(command::adjust_ghostbust, +0.01f);
+                controller::send_cmd(command::adjust_ghostbust, +0.01f);
                 break;
             case '<':
-                _controller.send_cmd(command::adjust_zoom, -0.1f);
+                controller::send_cmd(command::adjust_zoom, -0.1f);
                 break;
             case '>':
-                _controller.send_cmd(command::adjust_zoom, +0.1f);
+                controller::send_cmd(command::adjust_zoom, +0.1f);
                 break;
             case eq::KC_LEFT:
-                _controller.send_cmd(command::seek, -10.0f);
+                controller::send_cmd(command::seek, -10.0f);
                 break;
             case eq::KC_RIGHT:
-                _controller.send_cmd(command::seek, +10.0f);
-                break;
-            case eq::KC_UP:
-                _controller.send_cmd(command::seek, -60.0f);
+                controller::send_cmd(command::seek, +10.0f);
                 break;
             case eq::KC_DOWN:
-                _controller.send_cmd(command::seek, +60.0f);
+                controller::send_cmd(command::seek, -60.0f);
                 break;
-            case eq::KC_PAGE_UP:
-                _controller.send_cmd(command::seek, -600.0f);
+            case eq::KC_UP:
+                controller::send_cmd(command::seek, +60.0f);
                 break;
             case eq::KC_PAGE_DOWN:
-                _controller.send_cmd(command::seek, +600.0f);
+                controller::send_cmd(command::seek, -600.0f);
+                break;
+            case eq::KC_PAGE_UP:
+                controller::send_cmd(command::seek, +600.0f);
                 break;
             }
         }
@@ -702,16 +665,11 @@ public:
             // Event position relative to destination view (which seems to be the same as the canvas?)
             float dest = event->data.context.vp.x + event_x * event->data.context.vp.w;
             dest = std::min(std::max(dest, 0.0f), 1.0f);                // Clamp to [0,1] - just to be sure
-            _controller.send_cmd(command::set_pos, dest);               // Seek to this position
+            controller::send_cmd(command::set_pos, dest);               // Seek to this position
         }
         return true;
     }
 
-    player_eq_node *player()
-    {
-        return &_player;
-    }
-
 private:
     void compute_3d_canvas(float *height, float *distance)
     {
@@ -790,24 +748,23 @@ private:
 class eq_node : public eq::Node
 {
 private:
+    dispatch* _dispatch;
     bool _is_app_node;
     player_eq_node _player;
 
 public:
     eq_init_data init_data;
     eq_frame_data frame_data;
-    video_frame frame_template;
 
-    eq_node(eq::Config *parent) :
-        eq::Node(parent),
-        _is_app_node(false),
-        _player(),
-        init_data(),
-        frame_data(),
-        frame_template()
+    eq_node(eq::Config *parent) : eq::Node(parent), _dispatch(NULL), _is_app_node(false)
     {
     }
 
+    ~eq_node()
+    {
+        delete _dispatch;
+    }
+
 protected:
     virtual bool configInit(const eq::uint128_t &init_id)
     {
@@ -817,6 +774,7 @@ protected:
         }
         // Map our InitData instance to the master instance
         eq_config *config = static_cast<eq_config *>(getConfig());
+        config->setLatency( 0 );
         if (!config->mapObject(&init_data, init_id))
         {
             setError(ERROR_MAP_INITDATA_FAILED);
@@ -834,21 +792,19 @@ protected:
             return false;
         }
 
-        msg::set_level(init_data.init_data.log_level);
         msg::dbg(HERE);
         // Create decoders and input
         if (!_is_app_node)
         {
-            if (!_player.init(init_data.init_data, frame_template))
+            _dispatch = new dispatch(NULL, NULL, true, init_data.flat_screen, true,
+                    false, false, init_data.params.log_level(), init_data.params.benchmark(),
+                    init_data.params.swap_interval());
+            if (!_player.init(init_data.input))
             {
                 setError(ERROR_PLAYER_INIT_FAILED);
                 return false;
             }
         }
-        else
-        {
-            frame_template = config->frame_template;
-        }
         msg::dbg(HERE);
         return true;
     }
@@ -862,7 +818,8 @@ protected:
         // Unmap our InitData instance
         config->unmapObject(&init_data);
         // Cleanup
-        _player.close();
+        if (!_is_app_node)
+            _player.close();
         msg::dbg(HERE);
         return eq::Node::configExit();
     }
@@ -878,6 +835,7 @@ protected:
         }
         else
         {
+            _dispatch->load_state(frame_data.dispatch_state);
             if (frame_data.seek_to >= 0)
             {
                 _player.seek(frame_data.seek_to);
@@ -917,14 +875,9 @@ public:
     const video_frame &get_video_frame()
     {
         if (_is_app_node)
-        {
-            eq_config *config = static_cast<eq_config *>(getConfig());
-            return config->player()->get_video_frame();
-        }
+            return global_dispatch->get_player()->get_video_frame();
         else
-        {
             return _player.get_video_frame();
-        }
     }
 };
 
@@ -952,6 +905,7 @@ public:
     }
 
 protected:
+
     virtual bool configInitGL(const eq::uint128_t &init_id)
     {
         msg::dbg(HERE);
@@ -979,6 +933,13 @@ protected:
         msg::dbg(HERE);
         return eq::Window::configExitGL();
     }
+
+    virtual void swapBuffers()
+    {
+        eq_node *node = static_cast<eq_node *>(getNode());
+        if (node->frame_data.display_frame)
+            eq::Window::swapBuffers();
+    }
 };
 
 /*
@@ -1062,7 +1023,6 @@ protected:
     {
         // Get frame data via from the node
         eq_node *node = static_cast<eq_node *>(getNode());
-        _video_output.set_parameters(node->frame_data.params);
         // Do as we're told
         if (node->frame_data.prep_frame)
         {
@@ -1081,9 +1041,12 @@ protected:
         startFrame(frame_number);
     }
 
-    virtual void frameFinish(const eq::uint128_t &, const uint32_t frame_number)
+    void frameViewFinish( const eq::uint128_t& id )
     {
-        releaseFrame(frame_number);
+        eq_node *node = static_cast<eq_node *>(getNode());
+        if (node->frame_data.display_statistics)
+            drawStatistics();
+        eq::Channel::frameViewFinish( id );
     }
 };
 
@@ -1124,18 +1087,22 @@ public:
  * player_equalizer
  */
 
+static player_equalizer* global_player_equalizer = NULL;
+
 player_equalizer::player_equalizer(int *argc, char *argv[], bool flat_screen) :
-    player(player::slave), _flat_screen(flat_screen)
+    player(), _flat_screen(flat_screen)
 {
+    assert(!global_player_equalizer);
+    global_player_equalizer = this;
     /* Initialize Equalizer */
     initErrors();
-    _node_factory = static_cast<void *>(new eq_node_factory);
-    if (!eq::init(*argc, argv, static_cast<eq::NodeFactory *>(_node_factory)))
+    _node_factory = new eq_node_factory;
+    if (!eq::init(*argc, argv, _node_factory))
     {
         throw exc(_("Equalizer initialization failed."));
     }
     /* Get a configuration */
-    _config = static_cast<void *>(eq::getConfig(*argc, argv));
+    _config = static_cast<eq_config *>(eq::getConfig(*argc, argv));
     // The following code is only executed on the application node because
     // eq::getConfig() does not return on other nodes.
     if (!_config)
@@ -1146,32 +1113,32 @@ player_equalizer::player_equalizer(int *argc, char *argv[], bool flat_screen) :
 
 player_equalizer::~player_equalizer()
 {
-    delete static_cast<eq_node_factory *>(_node_factory);
+    delete _node_factory;
+    global_player_equalizer = NULL;
 }
 
-void player_equalizer::open(const player_init_data &init_data)
+void player_equalizer::open()
 {
-    eq_config *config = static_cast<eq_config *>(_config);
-    if (!config->init(init_data, _flat_screen))
+    if (!_config->init(*(global_dispatch->get_input_data()), _flat_screen))
     {
         throw exc(_("Equalizer configuration initialization failed."));
     }
 }
 
-void player_equalizer::run()
+void player_equalizer::close()
 {
-    eq_config *config = static_cast<eq_config *>(_config);
-    while (config->isRunning())
-    {
-        config->startFrame();
-        config->finishFrame();
-    }
-    config->exit();
-    eq::releaseConfig(config);
+    _config->exit();
+    eq::releaseConfig(_config);
     eq::exit();
     exitErrors();
 }
 
-void player_equalizer::close()
+void player_equalizer::mainloop()
 {
+    assert(global_player_equalizer);
+    eq_config* config = global_player_equalizer->_config;
+    while (config->isRunning()) {
+        config->startFrame();
+        config->finishFrame();
+    }
 }
