@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  * Joe <cuchac@email.cz>
@@ -30,7 +30,7 @@
 
 #include "media_data.h"
 #include "subtitle_renderer.h"
-#include "controller.h"
+#include "dispatch.h"
 
 
 class video_output : public controller
@@ -76,6 +76,19 @@ private:
     float _tex_coords[2][4][2];
 
 private:
+    // GL Helper functions
+    bool xglCheckError(const std::string& where = std::string()) const;
+    GLuint xglCompileShader(const std::string& name, GLenum type, const std::string& src) const;
+    GLuint xglCreateProgram(GLuint vshader, GLuint fshader) const;
+    GLuint xglCreateProgram(const std::string& name,
+            const std::string& vshader_src, const std::string& fshader_src) const;
+    void xglLinkProgram(const std::string& name, const GLuint prg) const;
+    void xglDeleteProgram(GLuint prg) const;
+
+    void draw_quad(float x, float y, float w, float h,
+            const float tex_coords[2][4][2] = NULL,
+            const float more_tex_coords[4][2] = NULL) const;
+
     // Step 1: initialize/deinitialize, and check if reinitialization is necessary
     void input_init(int index, const video_frame &frame);
     void input_deinit(int index);
@@ -96,32 +109,43 @@ private:
 protected:
     subtitle_renderer _subtitle_renderer;
 
+    virtual GLEWContext* glewGetContext() const = 0;
+
     // Get total size of the video display area. For single window output, this
     // is the same as the current viewport. The Equalizer video output can override
     // this function.
-    virtual int video_display_width();
-    virtual int video_display_height();
+    virtual int video_display_width() const;
+    virtual int video_display_height() const;
 
     virtual void make_context_current() = 0;    // Make sure our OpenGL context is current
-    virtual bool context_is_stereo() = 0;       // Is our current OpenGL context a stereo context?
+    virtual bool context_is_stereo() const = 0; // Is our current OpenGL context a stereo context?
     virtual void recreate_context(bool stereo) = 0;     // Recreate an OpenGL context and make it current
-    virtual void trigger_update() = 0;          // Trigger a redraw (i.e. make GL context current and call display())
     virtual void trigger_resize(int w, int h) = 0;      // Trigger a resize the video area
 
-    void clear();                               // Clear the video area
+    void clear() const;                         // Clear the video area
     void reshape(int w, int h);                 // Call this when the video area was resized
-    bool need_redisplay_on_move();              // Whether we need to redisplay if the video area moved
+
+    /* Get screen properties (fixed) */
+    virtual int screen_width() const = 0;       // in pixels
+    virtual int screen_height() const = 0;      // in pixels
+    virtual float screen_pixel_aspect_ratio() const = 0;// the aspect ratio of a pixel on screen
+
+    /* Get current video area properties */
+    virtual int width() const = 0;              // in pixels
+    virtual int height() const = 0;             // in pixels
+    virtual int pos_x() const = 0;              // in pixels
+    virtual int pos_y() const = 0;              // in pixels
 
     /* Display the current frame.
      * The first version is used by Equalizer, which needs to set some special properties.
      * The second version is for everyone else.
      * TODO: This function needs to handle interlaced frames! */
-    void display_current_frame(bool keep_viewport, bool mono_right_instead_of_left,
+    void display_current_frame(int64_t display_frameno, bool keep_viewport, bool mono_right_instead_of_left,
             float x, float y, float w, float h,
             const GLint viewport[2][4], const float tex_coords[2][4][2]);
-    void display_current_frame()
+    void display_current_frame(int64_t display_frameno = 0)
     {
-        display_current_frame(false, false, -1.0f, -1.0f, 2.0f, 2.0f, _viewport, _tex_coords);
+        display_current_frame(display_frameno, false, false, -1.0f, -1.0f, 2.0f, 2.0f, _viewport, _tex_coords);
     }
 
 public:
@@ -142,40 +166,24 @@ public:
     void set_suitable_size(int w, int h, float ar, parameters::stereo_mode_t stereo_mode);
 
     /* Get capabilities */
-    virtual bool supports_stereo() const = 0;   // Is OpenGL quad buffered stereo available?
-
-    /* Get screen properties (fixed) */
-    virtual int screen_width() = 0;             // in pixels
-    virtual int screen_height() = 0;            // in pixels
-    virtual float screen_pixel_aspect_ratio() = 0;      // the aspect ratio of a pixel on screen
-
-    /* Get current video area properties */
-    virtual int width() = 0;                    // in pixels
-    virtual int height() = 0;                   // in pixels
-    virtual int pos_x() = 0;                    // in pixels
-    virtual int pos_y() = 0;                    // in pixels
-    virtual bool fullscreen() = 0;              // whether the video area is in fullscreen mode
+    virtual bool supports_stereo() const = 0;           // Is OpenGL quad buffered stereo available?
 
     /* Center video area on screen */
     virtual void center() = 0;
 
     /* Enter/exit fullscreen mode */
-    virtual void enter_fullscreen(int screen) = 0;
+    virtual void enter_fullscreen() = 0;
     virtual void exit_fullscreen() = 0;
-    virtual bool toggle_fullscreen(int screen) = 0;
 
     /* Process window system events (if applicable) */
     virtual void process_events() = 0;
     
     /* Prepare a new frame for display. */
-    void prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle);
+    virtual void prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle);
     /* Switch to the next frame (make it the current one) */
-    void activate_next_frame();
-    /* Set display parameters. */
-    void set_parameters(const parameters &params);
-
-    /* Receive a notification from the player. */
-    virtual void receive_notification(const notification &note) = 0;
+    virtual void activate_next_frame();
+    /* Get an estimation of when the next frame will appear on screen */
+    virtual int64_t time_to_next_frame_presentation() const;
 };
 
 #endif
