@@ -23,15 +23,14 @@
 #include "config.h"
 
 #include <GL/glew.h>
-#ifdef GLEW_MX
-static GLEWContext _glewContext;
-static GLEWContext* glewGetContext() { return &_glewContext; }
-#endif
 
 #include <cmath>
 #include <cstdlib>
+#include <cerrno>
+#include <cstring>
 #include <unistd.h>
 
+#include <QCoreApplication>
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QGridLayout>
@@ -45,6 +44,7 @@ static GLEWContext* glewGetContext() { return &_glewContext; }
 #ifdef Q_WS_X11
 # include <QX11Info>
 # include <X11/Xlib.h>
+# include <GL/glxew.h>
 #endif
 
 #include "gettext.h"
@@ -60,67 +60,255 @@ static GLEWContext* glewGetContext() { return &_glewContext; }
 #include "lib_versions.h"
 
 
+/* The GL thread */
+
+gl_thread::gl_thread(video_output_qt* vo_qt, video_output_qt_widget* vo_qt_widget) :
+    _vo_qt(vo_qt), _vo_qt_widget(vo_qt_widget),
+    _render(false),
+    _activate_next_frame(false),
+    _resize(false),
+    _prepare_next_frame(false),
+    _have_prepared_frame(false),
+    _failure(false)
+{
+    for (int i = 0; i < 2; i++) {
+        for (int p = 0; p < 3; p++) {
+            _next_data[i][p] = NULL;
+            _next_line_size[i][p] = 0;
+        }
+    }
+}
+
+gl_thread::~gl_thread()
+{
+    for (int i = 0; i < 2; i++) {
+        for (int p = 0; p < 3; p++) {
+            std::free(_next_data[i][p]);
+        }
+    }
+}
+
+#ifdef Q_WS_X11
+GLXEWContext* gl_thread::glxewGetContext() const
+{
+    return _vo_qt->glxewGetContext();
+}
+#endif
+
+void gl_thread::set_render(bool r)
+{
+    _render = r;
+    _display_frameno = 0;
+    _pti = 0;
+    _ptc = 0;
+}
+
+void gl_thread::activate_next_frame()
+{
+    _activate_next_frame = true;
+}
+
+void gl_thread::resize(int w, int h)
+{
+    _w = w;
+    _h = h;
+    _resize = true;
+}
+
+void gl_thread::prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle)
+{
+    _prepare_next_mutex.lock();
+    _next_subtitle = subtitle;
+    _next_frame = frame;
+    // Copy the data because the decoder can overwrite the buffers at any time
+    try {
+        if (_next_frame.is_valid()) {
+            for (int i = 0; i < 2; i++) {
+                for (int p = 0; p < 3; p++) {
+                    size_t height = _next_frame.raw_height;
+                    if (_next_frame.layout == video_frame::yuv420p && p > 0)
+                        height /= 2;
+                    size_t data_size = height * _next_frame.line_size[i][p];
+                    if (_next_line_size[i][p] != _next_frame.line_size[i][p]) {
+                        _next_data[i][p] = std::realloc(_next_data[i][p], data_size);
+                        if (data_size > 0 && !_next_data[i][p]) {
+                            throw exc(ENOMEM);
+                        }
+                        _next_line_size[i][p] = _next_frame.line_size[i][p];
+                    }
+                    if (data_size > 0) {
+                        std::memcpy(_next_data[i][p], _next_frame.data[i][p], data_size);
+                    }
+                    _next_frame.data[i][p] = _next_data[i][p];
+                }
+            }
+        }
+    }
+    catch (std::exception& e) {
+        _e = e;
+        _render = false;
+        _failure = true;
+    }
+    _prepare_next_mutex.unlock();
+    _prepare_next_frame = true;
+}
+
+void gl_thread::run()
+{
+    try {
+        while (_render) {
+            assert(_vo_qt_widget->context()->isValid());
+            _vo_qt_widget->makeCurrent();
+            if (QGLContext::currentContext() == _vo_qt_widget->context()) {
+                if (_activate_next_frame && _have_prepared_frame) {
+                    _vo_qt->video_output::activate_next_frame();
+                    _have_prepared_frame = false;
+                    _activate_next_frame = false;
+                }
+                if (_resize) {
+                    _vo_qt->reshape(_w, _h);
+                    _resize = false;
+                }
+#ifdef Q_WS_X11
+                GLuint counter;
+                if (GLXEW_SGI_video_sync && glXGetVideoSyncSGI(&counter) == 0)
+                    _display_frameno = counter;
+                else
+                    _display_frameno++;
+#else
+                _display_frameno++;
+#endif
+                _vo_qt->display_current_frame(_display_frameno);
+                if (_prepare_next_frame) {
+                    _prepare_next_mutex.lock();
+                    _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
+                    _prepare_next_mutex.unlock();
+                    _prepare_next_frame = false;
+                    _have_prepared_frame = true;
+                }
+            }
+            _vo_qt_widget->swapBuffers();
+            // When the buffer swap returns, the current frame is just now presented on screen.
+            _pt_mutex.lock();
+            _pt[_pti] = timer::get_microseconds(timer::monotonic);
+            _pti++;
+            if (_pti >= _pts)
+                _pti = 0;
+            if (_ptc < _pts)
+                _ptc++;
+            _pt_mutex.unlock();
+        }
+    }
+    catch (std::exception& e) {
+        _e = e;
+        _render = false;
+        _failure = true;
+    }
+}
+
+int64_t gl_thread::time_to_next_frame_presentation()
+{
+    if (_ptc < _pts)    // no reliable data yet; assume immediate display
+        return 0;
+    _pt_mutex.lock();
+    int last_pti = _pti == 0 ? _pts - 1 : _pti - 1;
+    int64_t last_pt = _pt[last_pti];
+    int64_t presentation_duration = 0;
+    for (int i = 0; i < _pts - 1; i++) {
+        int cpti = _pti - i - 1;
+        if (cpti < 0)
+            cpti += _pts;
+        int ppti = cpti - 1;
+        if (ppti < 0)
+            ppti = _pts - 1;
+        assert(_pt[cpti] - _pt[ppti] >= 0);
+        presentation_duration += _pt[cpti] - _pt[ppti];
+    }
+    presentation_duration /= _pts - 1;
+    _pt_mutex.unlock();
+    int64_t now = timer::get_microseconds(timer::monotonic);
+    return last_pt + presentation_duration - now;
+}
+
 /* The GL widget */
 
 video_output_qt_widget::video_output_qt_widget(
         video_output_qt *vo, const QGLFormat &format, QWidget *parent) :
-    QGLWidget(format, parent), _vo(vo)
+    QGLWidget(format, parent), _vo(vo), _gl_thread(vo, this),
+    _width(0), _height(0), _pos_x(0), _pos_y(0)
 {
+    setAutoBufferSwap(false);
     setFocusPolicy(Qt::StrongFocus);
+    connect(&_timer, SIGNAL(timeout()), this, SLOT(check_gl_thread()));
 }
 
-video_output_qt_widget::~video_output_qt_widget()
+int video_output_qt_widget::vo_width() const
 {
+    return _width;
 }
 
-void video_output_qt_widget::move_event()
+int video_output_qt_widget::vo_height() const
 {
-    if (_vo->need_redisplay_on_move())
-    {
-        update();
-    }
+    return _height;
 }
 
-void video_output_qt_widget::paintGL()
+int video_output_qt_widget::vo_pos_x() const
 {
-    try
-    {
-        _vo->display_current_frame();
-    }
-    catch (std::exception &e)
-    {
-        QMessageBox::critical(this, _("Error"), e.what());
-        // Disable further output and stop the player
-        _vo->prepare_next_frame(video_frame(), subtitle_box());
-        _vo->activate_next_frame();
+    return _pos_x;
+}
+
+int video_output_qt_widget::vo_pos_y() const
+{
+    return _pos_y;
+}
+
+void video_output_qt_widget::check_gl_thread()
+{
+    // XXX We need to know our current position on the screen (global pixel coordinates).
+    // Querying our position in vo_pos_*() does not work since that function is called
+    // from the GL thread and mapToGlocal() seems to block for some strange reason.
+    // Therefore we record our current position here.
+    _pos_x = mapToGlobal(QPoint(0, 0)).x();
+    _pos_y = mapToGlobal(QPoint(0, 0)).y();
+    if (_gl_thread.failure()) {
+        stop_rendering();
+        QMessageBox::critical(this, _("Error"), _gl_thread.exception().what());
         _vo->send_cmd(command::toggle_play);
     }
 }
 
-void video_output_qt_widget::resizeGL(int w, int h)
+void video_output_qt_widget::start_rendering()
 {
-    _vo->reshape(w, h);
+    QApplication::syncX();
+    _gl_thread.set_render(true);
+    _gl_thread.start();
+    _timer.start(0);
 }
 
-void video_output_qt_widget::moveEvent(QMoveEvent *)
+void video_output_qt_widget::stop_rendering()
 {
-    move_event();
+    _gl_thread.set_render(false);
+    _gl_thread.wait();
+    _timer.stop();
+    QApplication::syncX();
+}
+
+void video_output_qt_widget::resizeEvent(QResizeEvent* event)
+{
+    _width = event->size().width();
+    _height = event->size().height();
+    _gl_thread.resize(_width, _height);
 }
 
 void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
 {
-    if (_vo->_container_is_external && !_vo->_playing)
-    {
-        QGLWidget::keyPressEvent(event);
-        return;
-    }
     switch (event->key())
     {
     case Qt::Key_Escape:
         // ESC should stop playback, unless we're in fullscreen mode.
         // In that case, it should simply leave fullscreen mode, since
         // this is what most users expect.
-        if (_vo->_fullscreen)
+        if (dispatch::parameters().fullscreen())
         {
             _vo->send_cmd(command::toggle_fullscreen);
         }
@@ -151,18 +339,20 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
 #endif
         _vo->send_cmd(command::toggle_pause);
         break;
+#if QT_VERSION >= 0x040700
     case Qt::Key_MediaPlay:
-        if (_vo->_pausing)
+        if (dispatch::pausing())
         {
             _vo->send_cmd(command::toggle_pause);
         }
         break;
     case Qt::Key_MediaPause:
-        if (!_vo->_pausing)
+        if (!dispatch::pausing())
         {
             _vo->send_cmd(command::toggle_pause);
         }
         break;
+#endif
     case Qt::Key_V:
         _vo->send_cmd(command::cycle_video_stream);
         break;
@@ -240,16 +430,16 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
     case Qt::Key_Right:
         _vo->send_cmd(command::seek, +10.0f);
         break;
-    case Qt::Key_Up:
+    case Qt::Key_Down:
         _vo->send_cmd(command::seek, -60.0f);
         break;
-    case Qt::Key_Down:
+    case Qt::Key_Up:
         _vo->send_cmd(command::seek, +60.0f);
         break;
-    case Qt::Key_PageUp:
+    case Qt::Key_PageDown:
         _vo->send_cmd(command::seek, -600.0f);
         break;
-    case Qt::Key_PageDown:
+    case Qt::Key_PageUp:
         _vo->send_cmd(command::seek, +600.0f);
         break;
     default:
@@ -291,22 +481,50 @@ video_container_widget::video_container_widget(QWidget *parent) : QWidget(parent
     setAutoFillBackground(true);
 }
 
+void video_container_widget::start_timer()
+{
+    _timer = new QTimer(this);
+    connect(_timer, SIGNAL(timeout()), this, SLOT(playloop_step()));
+    _timer->start(0);
+}
+
+void video_container_widget::playloop_step()
+{
+    try {
+        dispatch::step();
+        dispatch::process_all_events();
+    }
+    catch (std::exception& e) {
+        send_cmd(command::close);
+        QMessageBox::critical(this, "Error", e.what());
+    }
+}
+
+void video_container_widget::receive_notification(const notification& note)
+{
+    if (note.type == notification::play && dispatch::playing()) {
+        grab_focus();
+    } else if (note.type == notification::quit && _timer) {
+        QApplication::quit();
+    }
+}
+
 void video_container_widget::set_recommended_size(int w, int h)
 {
     _w = w;
     _h = h;
 }
 
+void video_container_widget::grab_focus()
+{
+    childAt(0, 0)->setFocus(Qt::OtherFocusReason);
+}
+
 QSize video_container_widget::sizeHint() const
 {
     return QSize(_w, _h);
 }
 
-void video_container_widget::moveEvent(QMoveEvent *)
-{
-    emit move_event();
-}
-
 void video_container_widget::closeEvent(QCloseEvent *)
 {
     send_cmd(command::toggle_play);
@@ -315,22 +533,38 @@ void video_container_widget::closeEvent(QCloseEvent *)
 
 /* The video_output_qt class */
 
-video_output_qt::video_output_qt(const int swap_interval, video_container_widget *container_widget) :
+video_output_qt::video_output_qt(video_container_widget *container_widget) :
     video_output(),
     _container_widget(container_widget),
     _container_is_external(container_widget != NULL),
     _widget(NULL),
     _fullscreen(false),
-    _playing(false),
-    _pausing(false)
+    _screensaver_inhibited(false),
+    _recreate_context(false),
+    _recreate_context_stereo(false)
 {
     if (!_container_widget)
     {
         _container_widget = new video_container_widget(NULL);
+        _container_widget->start_timer();
     }
     _format.setDoubleBuffer(true);
-    _format.setSwapInterval(swap_interval);
+    _format.setSwapInterval(dispatch::parameters().swap_interval());
     _format.setStereo(false);
+    // Save some constant values into member variables
+    // so that we don't have to call Qt functions when the GL thread
+    // calls our screen_*() functions.
+    _screen_width = QApplication::desktop()->screenGeometry().width();
+    _screen_height = QApplication::desktop()->screenGeometry().height();
+    _screen_pixel_aspect_ratio =
+        static_cast<float>(QApplication::desktop()->logicalDpiY())
+        / static_cast<float>(QApplication::desktop()->logicalDpiX());
+    if (std::fabs(_screen_pixel_aspect_ratio - 1.0f) < 0.03f) {
+        // This screen most probably has square pixels, and the difference to 1.0
+        // is only due to slightly inaccurate measurements and rounding. Force
+        // 1.0 so that the user gets expected results.
+        _screen_pixel_aspect_ratio = 1.0f;
+    }
 }
 
 video_output_qt::~video_output_qt()
@@ -350,6 +584,12 @@ void video_output_qt::init()
         _widget->makeCurrent();
         set_opengl_versions();
         GLenum err = glewInit();
+#ifdef Q_WS_X11
+        if (err == GLEW_OK)
+        {
+            err = glxewInit();
+        }
+#endif
         if (err != GLEW_OK)
         {
             throw exc(str::asprintf(_("Cannot initialize GLEW: %s"),
@@ -365,6 +605,9 @@ void video_output_qt::init()
         // Initialize GL things
         glMatrixMode(GL_PROJECTION);
         glLoadIdentity();
+        // Start rendering
+        _widget->doneCurrent();
+        _widget->start_rendering();
     }
 }
 
@@ -378,7 +621,7 @@ int64_t video_output_qt::wait_for_subtitle_renderer()
     exc init_exception;
     QDialog *mbox = NULL;
     // Show a dialog only in GUI mode
-    if (_container_is_external && !_fullscreen)
+    if (_container_is_external && !dispatch::parameters().fullscreen())
     {
         mbox = new QDialog(_container_widget);
         mbox->setModal(true);
@@ -424,9 +667,9 @@ void video_output_qt::deinit()
 {
     if (_widget)
     {
+        _widget->stop_rendering();
         _widget->makeCurrent();
         video_output::deinit();
-        _widget->doneCurrent();
         delete _widget;
         _widget = NULL;
     }
@@ -440,7 +683,6 @@ void video_output_qt::create_widget()
         QMessageBox::critical(_widget, _("Error"), _("Cannot get valid OpenGL context."));
         std::exit(1);
     }
-    QObject::connect(_container_widget, SIGNAL(move_event()), _widget, SLOT(move_event()));
     if ((_format.doubleBuffer() && !_widget->format().doubleBuffer())
             || (_format.stereo() && !_widget->format().stereo()))
     {
@@ -481,47 +723,52 @@ void video_output_qt::create_widget()
     process_events();
 }
 
+#ifdef Q_WS_X11
+GLXEWContext* video_output_qt::glxewGetContext() const
+{
+    return const_cast<GLXEWContext*>(&_glxew_context);
+}
+#endif
+
+GLEWContext* video_output_qt::glewGetContext() const
+{
+    return const_cast<GLEWContext*>(&_glew_context);
+}
+
 void video_output_qt::make_context_current()
 {
     _widget->makeCurrent();
 }
 
-bool video_output_qt::context_is_stereo()
+bool video_output_qt::context_is_stereo() const
 {
     return (_format.stereo());
 }
 
 void video_output_qt::recreate_context(bool stereo)
 {
-    deinit();
-    _format.setStereo(stereo);
-    init();
-    clear();
-}
-
-void video_output_qt::trigger_update()
-{
-    _widget->update();
+    // This was called from the GL thread (from inside video_output).
+    // The request will be handled the next time process_events() is run,
+    // because it involves destroying the current GL context and thread.
+    _recreate_context = true;
+    _recreate_context_stereo = stereo;
 }
 
 void video_output_qt::trigger_resize(int w, int h)
 {
     _container_widget->set_recommended_size(w, h);
     _container_widget->updateGeometry();
-    if (!_container_is_external)
-    {
-        _container_widget->adjustSize();
-    }
+    _container_widget->adjustSize();
 }
 
 void video_output_qt::mouse_set_pos(float dest)
 {
-    if (_fullscreen || _container_is_external)
+    if (dispatch::parameters().fullscreen() || _container_is_external)
     {
         // Disabled in fullscreen and GUI mode
         return;
     }
-    if (_playing)
+    if (dispatch::playing())
     {
         send_cmd(command::set_pos, dest);
     }
@@ -534,7 +781,7 @@ void video_output_qt::mouse_toggle_fullscreen()
         // Disabled in non-GUI mode
         return;
     }
-    if (_playing)
+    if (dispatch::playing())
     {
         send_cmd(command::toggle_fullscreen);
     }
@@ -570,22 +817,6 @@ void video_output_qt::resume_screensaver()
 #endif
 }
 
-void video_output_qt::move_event()
-{
-    if (_widget)
-    {
-        _widget->move_event();
-    }
-}
-
-void video_output_qt::grab_focus()
-{
-    if (_widget)
-    {
-        _widget->setFocus(Qt::OtherFocusReason);
-    }
-}
-
 bool video_output_qt::supports_stereo() const
 {
     QGLFormat fmt = _format;
@@ -596,59 +827,44 @@ bool video_output_qt::supports_stereo() const
     return ret;
 }
 
-int video_output_qt::screen_width()
+int video_output_qt::screen_width() const
 {
-    return QApplication::desktop()->screenGeometry().width();
+    return _screen_width;
 }
 
-int video_output_qt::screen_height()
+int video_output_qt::screen_height() const
 {
-    return QApplication::desktop()->screenGeometry().height();
+    return _screen_height;
 }
 
-float video_output_qt::screen_pixel_aspect_ratio()
+float video_output_qt::screen_pixel_aspect_ratio() const
 {
-    float screen_pixel_ar =
-        static_cast<float>(QApplication::desktop()->logicalDpiY())
-        / static_cast<float>(QApplication::desktop()->logicalDpiX());
-    if (std::fabs(screen_pixel_ar - 1.0f) < 0.03f)
-    {
-        // This screen most probably has square pixels, and the difference to 1.0
-        // is only due to slightly inaccurate measurements and rounding. Force
-        // 1.0 so that the user gets expected results.
-        screen_pixel_ar = 1.0f;
-    }
-    return screen_pixel_ar;
+    return _screen_pixel_aspect_ratio;
 }
 
-int video_output_qt::width()
+int video_output_qt::width() const
 {
-    return _widget->width();
+    return _widget->vo_width();
 }
 
-int video_output_qt::height()
+int video_output_qt::height() const
 {
-    return _widget->height();
+    return _widget->vo_height();
 }
 
-int video_output_qt::pos_x()
+int video_output_qt::pos_x() const
 {
-    return _widget->mapToGlobal(QPoint(0, 0)).x();
+    return _widget->vo_pos_x();
 }
 
-int video_output_qt::pos_y()
+int video_output_qt::pos_y() const
 {
-    return _widget->mapToGlobal(QPoint(0, 0)).y();
-}
-
-bool video_output_qt::fullscreen()
-{
-    return _fullscreen;
+    return _widget->vo_pos_y();
 }
 
 void video_output_qt::center()
 {
-    if (!_fullscreen)
+    if (!dispatch::parameters().fullscreen())
     {
         // Move the window, not the widget, so that this also works inside the GUI.
         int dest_screen_pos_x = (screen_width() - width()) / 2;
@@ -662,11 +878,11 @@ void video_output_qt::center()
     }
 }
 
-void video_output_qt::enter_fullscreen(int screens)
+void video_output_qt::enter_fullscreen()
 {
     if (!_fullscreen)
     {
-        _fullscreen = true;
+        int screens = dispatch::parameters().fullscreen_screens();
         if (_container_is_external)
         {
             _container_widget->setWindowFlags(Qt::Window);
@@ -729,34 +945,27 @@ void video_output_qt::enter_fullscreen(int screens)
             XFlush(QX11Info::display());
 #endif
         }
-        grab_focus();
+        _container_widget->grab_focus();
         // Suspend the screensaver after going fullscreen, so that our window ID
         // represents the fullscreen window. We need to have the same ID for resume.
-        suspend_screensaver();
+        if (dispatch::parameters().fullscreen_inhibit_screensaver()) {
+            suspend_screensaver();
+            _screensaver_inhibited = true;
+        }
+        _fullscreen = true;
     }
 }
 
-bool video_output_qt::toggle_fullscreen(int screens)
-{
-    if (_fullscreen)
-    {
-        exit_fullscreen();
-    }
-    else
-    {
-        enter_fullscreen(screens);
-    }
-    return _fullscreen;
-}
-
 void video_output_qt::exit_fullscreen()
 {
     if (_fullscreen)
     {
         // Resume the screensaver before disabling fullscreen, so that our window ID
         // still represents the fullscreen window and was the same when suspending the screensaver.
-        resume_screensaver();
-        _fullscreen = false;
+        if (_screensaver_inhibited) {
+            resume_screensaver();
+            _screensaver_inhibited = false;
+        }
         if (_container_is_external)
         {
             _container_widget->setWindowFlags(Qt::Widget);
@@ -768,28 +977,47 @@ void video_output_qt::exit_fullscreen()
         _container_widget->setWindowState(_widget->windowState() & ~Qt::WindowFullScreen);
         _container_widget->setCursor(Qt::ArrowCursor);
         _container_widget->show();
-        grab_focus();
+        _container_widget->raise();
+        _container_widget->grab_focus();
+        _fullscreen = false;
     }
 }
 
+void video_output_qt::prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle)
+{
+    if (_widget)
+        _widget->gl_thread()->prepare_next_frame(frame, subtitle);
+}
+
+void video_output_qt::activate_next_frame()
+{
+    if (_widget)
+        _widget->gl_thread()->activate_next_frame();
+}
+
+int64_t video_output_qt::time_to_next_frame_presentation() const
+{
+    return (_widget ? _widget->gl_thread()->time_to_next_frame_presentation() : 0);
+}
+
 void video_output_qt::process_events()
 {
+    if (_recreate_context) {
+        // To prevent recursion, we must unset this flag first: deinit() and init()
+        // cause more calls to process_events().
+        _widget->stop_rendering();
+        _recreate_context = false;
+        deinit();
+        _format.setStereo(_recreate_context_stereo);
+        init();
+    }
     QApplication::sendPostedEvents();
     QApplication::processEvents();
 }
 
-void video_output_qt::receive_notification(const notification &note)
+void video_output_qt::receive_notification(const notification& note)
 {
-    if (note.type == notification::play)
-    {
-        std::istringstream current(note.current);
-        s11n::load(current, _playing);
+    if (note.type == notification::play && !dispatch::playing()) {
+        exit_fullscreen();
     }
-    else if (note.type == notification::pause)
-    {
-        std::istringstream current(note.current);
-        s11n::load(current, _pausing);
-    }
-    /* More is currently not implemented.
-     * In the future, an on-screen display might show hints about what happened. */
 }
